import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { AgentService } from '../services/agent.service';

@Component({
  selector: 'app-agent',
  templateUrl: './agent.component.html',
  styleUrls: ['./agent.component.css'],
})
export class AgentComponent {
  @ViewChild('manageAgent') manageAgentModal;
  bsModalRef: BsModalRef;

  type: any = 'add';

  countries: any[];
  selectedCountry = 'Select Country';

  agents: any[];
  selectedAgent = 'Select Agent';

  allAgents: any = [];

  agentTypeList: any = [];
  agentCountry: any[];
  cities: any = [];
  locations: any = [];
  settlementCurrencies: any = [];
  agentCurrencies: any = [];
  modeofPayments: any = [];
  domesticTransactionTypes: any = [];
  internationalTransactionTypes: any = [];

  listofCurrencies: any = [];
  trAmountLimits: any = [];
  trAmountLimitBank: any = [];
  trAmountLimitDraft: any = [];
  trAmountLimitWalt: any = [];
  listofModeofPayments: any = [];
  rebateCurrencyType: any = [];
  rebateType: any = [
    {
      Id: 1,
      name: 'Fixed',
    },
    {
      Id: 2,
      name: 'Percentage',
    },
  ];

  commissionType: any = [
    {
      Id: 1,
      name: 'None',
    },
    {
      Id: 2,
      name: 'Both(+/-)',
    },
    {
      Id: 3,
      name: 'Plus(+)',
    },
    {
      Id: 4,
      name: 'Minus(-)',
    },
  ];

  manageAgentForm = new FormGroup({
    AgentId: new FormControl(''),
    CountryID: new FormControl('None', Validators.required),
    CityID: new FormControl('None', Validators.required),
    LocationID: new FormControl('None', Validators.required),
    Agent_Name: new FormControl('', Validators.required),
    Agent_Add: new FormControl('', Validators.required),
    ContactPerson: new FormControl(''),
    Ph: new FormControl(''),
    Fax: new FormControl(''),
    Email: new FormControl(''),
    SettlementCurrency: new FormControl('None', Validators.required),
    Base_Currency_Id: new FormControl('None', Validators.required),
    AllowedDomesticTransactionType: new FormControl('None'),
    AllowedInternationalTransactionType: new FormControl('None'),
    CurrenciesTrAmtLimit: new FormControl(''),
    CurrenciesTrAmtLimit_Bank: new FormControl(''),
    CurrenciesTrAmtLimit_DD: new FormControl(''),
    CurrenciesTrAmtLimit_WALT: new FormControl(''),
    ModeOfPaymentID: new FormControl(''),
    CreditLimitInUSD: new FormControl(''),
    TELLimit: new FormControl(''),
    NostroCode: new FormControl('', Validators.required),
    NostroAcc: new FormControl('', Validators.required),
    Rebate: new FormControl(false, Validators.required),
    TrdagentId: new FormControl('None', Validators.required),
    Trdtype: new FormControl(''),
    Trdtxnlimit: new FormControl(''),
    Trdtotalshare: new FormControl(''),
    Trdagencyshare: new FormControl(''),
    Trdpayoutagentshare: new FormControl(''),
    TrdcurrencyId: new FormControl(''),
    IsCommMargin: new FormControl(true, Validators.required),
    IsRTGSNEFT: new FormControl(true, Validators.required),
    CommMarginType: new FormControl(1),
    CTAonHold: new FormControl(false),
    IsActive: new FormControl(false),
  });

  constructor(
    private agentService: AgentService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getCountries();
    this.getAgentTypeList();
    this.getAgentSettlementCurrency();
    this.getRebateCurrencyType();
  }

  getCountries(userId = 0) {
    this.agentService
      .getCountries(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.countries = res;
        },
        error: (err) => {
          this.countries = [];
        },
        complete: () => {},
      });
  }

  onCountryChange() {
    if (this.selectedCountry != 'Select Country') {
      this.getAgents(this.selectedCountry);
    }
  }

  getAgents(countryId: any = 0) {
    this.agentService
      .getAgents(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.agents = res;
        },
        error: (err) => {
          this.agents = [];
        },
        complete: () => {},
      });
  }

  getAllAgents(countryId = 0, agentId = 0) {
    this.agentService
      .getAllAgents(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.allAgents = res;
        },
        error: (err) => {
          this.allAgents = [];
        },
        complete: () => {},
      });
  }

  searchAgents() {
    let countryId: any = 0;
    let agentId: any = 0;

    if (this.selectedCountry !== 'Select Country') {
      countryId = this.selectedCountry;
    }

    if (this.selectedAgent !== 'Select Agent') {
      agentId = this.selectedAgent;
    }
    this.getAllAgents(countryId, agentId);
  }

  getAgentTypeList() {
    this.agentService
      .getAgentTypeList(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.agentTypeList = res;
          if (this.type == 'add') {
            this.getCountryInsert(res?.[0]?.Group_Id);
          }
        },
        error: (err) => {
          this.agentTypeList = [];
        },
        complete: () => {},
      });
  }

  onAgentTypeChange(agentId: any) {
    if (agentId != 'None') {
      this.getCountryInsert(agentId);
    }
  }

  getCountryInsert(agentId: any = 0, response: any = '') {
    this.agentService
      .getCountryInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.agentCountry = res;
          if (response) {
            // this.setFormValues(response);
            this.getCity(0, agentId, response);
          } else {
            this.getCity(0);
          }
        },
        error: (err) => {
          this.agentCountry = [];
        },
        complete: () => {},
      });
  }

  onCountryInsertChange(countryId: any) {
    if (countryId != 'None') {
      this.getCity(countryId);
      this.getAgentCurrency(countryId);
      this.getModeofPayment(countryId);
      this.getDomesticTransactionType(countryId, this.f.TrdagentId);
      this.getInternationalTransactionType(countryId, this.f.TrdagentId);
    }
  }

  getCity(countryId: any = 0, agentId: any = '', response: any = '') {
    this.agentService
      .getCity(getCacheData('admin_token'), getCacheData('username'), countryId)
      .subscribe({
        next: (res: any) => {
          this.cities = res;
          if (response && this.type == 'view') {
            const countryId = this.agentCountry.find(
              (p) => p.CountryName == response?.[0]?.CountryName
            )?.CountryId;
            const cityId = this.cities.find(
              (p) => p.CityName == response?.[0]?.CityName
            )?.CityID;
            this.getLocation(cityId, countryId);
            this.getAgentCurrency(countryId);
            this.getModeofPayment(countryId);
            this.getDomesticTransactionType(countryId, agentId);
            this.getInternationalTransactionType(countryId, agentId, response);
          } else if (this.type == 'edit' && response) {
            this.getLocation(response?.[0]?.CityID, response?.[0]?.CountryID);
            this.getAgentCurrency(response?.[0]?.CountryID);
            this.getModeofPayment(response?.[0]?.CountryID);
            this.getDomesticTransactionType(
              response?.[0]?.CountryID,
              response?.[0]?.AgentID
            );
            this.getInternationalTransactionType(
              response?.[0]?.CountryID,
              response?.[0]?.AgentID,
              response
            );
            // if (this.f.CountryID == 'None') {
            //   this.getLocation(
            //     res?.[0]?.CityID,
            //     this.agentCountry?.[0]?.CountryId
            //   );
            //   this.getAgentCurrency(this.agentCountry?.[0]?.CountryId);
            //   this.getModeofPayment(this.agentCountry?.[0]?.CountryId);
            //   this.getDomesticTransactionType(
            //     this.agentCountry?.[0]?.CountryId,
            //     this.agentTypeList?.[0]?.Group_Id
            //   );
            //   this.getInternationalTransactionType(
            //     this.agentCountry?.[0]?.CountryId,
            //     this.agentTypeList?.[0]?.Group_Id
            //   );
            // }
          }
        },
        error: (err) => {
          this.cities = [];
        },
        complete: () => {},
      });
  }

  onCityChange(cityId: any) {
    if (cityId != 'None') {
      this.getLocation(cityId, this.f.CountryID);
    }
  }

  getLocation(cityId: any = 0, countryId: any = 0) {
    this.agentService
      .getLocation(
        getCacheData('admin_token'),
        getCacheData('username'),
        cityId,
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.locations = res;
        },
        error: (err) => {
          this.locations = [];
        },
        complete: () => {},
      });
  }

  getAgentSettlementCurrency() {
    this.agentService
      .getAgentSettlementCurrency(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.settlementCurrencies = res;
        },
        error: (err) => {
          this.settlementCurrencies = [];
        },
        complete: () => {},
      });
  }

  getAgentCurrency(countryId: any = 0) {
    this.agentService
      .getAgentCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.agentCurrencies = res;
        },
        error: (err) => {
          this.agentCurrencies = [];
        },
        complete: () => {},
      });
  }

  getModeofPayment(countryId: any = 0) {
    this.agentService
      .getModeofPayment(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPayments = res;
        },
        error: (err) => {
          this.modeofPayments = [];
        },
        complete: () => {},
      });
  }

  getDomesticTransactionType(countryId: any = 0, agentType: any = 0) {
    this.agentService
      .getDomesticTransactionType(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        agentType
      )
      .subscribe({
        next: (res: any) => {
          this.domesticTransactionTypes = res;
        },
        error: (err) => {
          this.domesticTransactionTypes = [];
        },
        complete: () => {},
      });
  }

  getInternationalTransactionType(
    countryId: any = 0,
    agentType: any = 0,
    response: any = ''
  ) {
    this.agentService
      .getInternationalTransactionType(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        agentType
      )
      .subscribe({
        next: (res: any) => {
          this.internationalTransactionTypes = res;
          if (response) {
            this.setFormValues(response);
          }
        },
        error: (err) => {
          this.internationalTransactionTypes = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  openAgentModal(type: any) {
    if (type == 'add') {
      this.manageAgentForm.reset({
        TrdagentId: 'None',
        CountryID: 'None',
        CityID: 'None',
        LocationID: 'None',
        SettlementCurrency: 'None',
        Base_Currency_Id: 'None',
        AllowedDomesticTransactionType: 'None',
        AllowedInternationalTransactionType: 'None',
        ModeOfPaymentID: 'All',
        Trdtype: 'Fixed',
        CommMarginType: 1,
      });
    }
    this.type = type;
    this.openModal(this.manageAgentModal);
  }

  get f() {
    return this.manageAgentForm.value;
  }

  addAgent() {
    if (!this.manageAgentForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (!this.f.CurrenciesTrAmtLimit && this.trAmountLimits.length == 0) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      !this.f.CurrenciesTrAmtLimit_Bank &&
      this.trAmountLimitBank.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      !this.f.CurrenciesTrAmtLimit_DD &&
      this.trAmountLimitDraft.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      !this.f.CurrenciesTrAmtLimit_WALT &&
      this.trAmountLimitWalt.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (!this.f.ModeOfPaymentID && this.listofModeofPayments.length == 0) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (this.f.Base_Currency_Id == 'None') {
      alert('Please Select Currency.');
      return;
    }
    // if (this.f.AllowedDomesticTransactionType == 'None') {
    //   alert('Please Select Allowed Domestic Transaction Type.');
    //   return;
    // }
    if (
      this.f.AllowedInternationalTransactionType == 'None' &&
      this.f.AllowedDomesticTransactionType == 'None'
    ) {
      alert(
        'Please Select Allowed International or Domestic Transaction Type.'
      );
      return;
    }
    if (this.f.AllowedInternationalTransactionType == 'None') {
      alert('Please Select Allowed International Transaction Type.');
      return;
    }

    let isActive = '1';
    if (!this.f.IsActive) {
      isActive = '0';
    }

    let payload;
    payload = {
      AgentId: this.f.AgentId,
      CountryID: this.f.CountryID,
      CityID: this.f.CityID,
      LocationID: this.f.LocationID,
      Ex_Branch_ID: 1001,
      Agent_Name: this.f.Agent_Name,
      Agent_Add: this.f.Agent_Add,
      Ph: this.f.Ph,
      Fax: this.f.Fax,
      Email: this.f.Email,
      Base_Currency_Id: this.f.Base_Currency_Id,
      CreditLimitInUSD: Number(this.f.CreditLimitInUSD),
      TELLimit: Number(this.f.TELLimit),
      IndividualTransactionLimit: 0,
      AllowedDomesticTransactionType: 0,
      AllowedInternationalTransactionType:
        this.f.AllowedInternationalTransactionType,
      ContactPerson: this.f.ContactPerson,
      User_Group: 9,
      DefaultPassword: 'target@123',
      PasswordExpiryDuration: 30,
      SettlementCurrency: this.f.SettlementCurrency,
      Currencies: this.listofCurrencies,
      ModeOfPayment: this.listofModeofPayments,
      CurrenciesTrAmtLimit: this.trAmountLimits,
      CurrenciesTrAmtLimit_Bank: this.trAmountLimitBank,
      CurrenciesTrAmtLimit_DD: this.trAmountLimitDraft,
      CurrenciesTrAmtLimit_WALT: this.trAmountLimitWalt,
      COConHold: false,
      CTAonHold: this.f.CTAonHold ? this.f.CTAonHold : false,
      IsCommMargin: this.f.IsCommMargin ? this.f.IsCommMargin : false,
      IsRTGSNEFT: this.f.IsRTGSNEFT ? this.f.IsCommMargin : false,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy: '1001000',
      CommMarginType: 1,
      Rebate: false,
      TrdagentId: this.f.TrdagentId,
      Trdtype: 'F',
      Trdtxnlimit: 0,
      Trdtotalshare: 0,
      Trdagencyshare: 0,
      Trdpayoutagentshare: 0,
      TrdcurrencyId: 0,
      IsActive: isActive,
      NostroCode: Number(this.f.NostroCode),
      NostroAcc: Number(this.f.NostroAcc),
      RAPrefix: '',
    };

    if (this.f.Rebate) {
      payload = {
        ...payload,
        Rebate: true,
        TrdagentId: this.f.TrdagentId,
        Trdtype: this.f.Trdtype,
        Trdtxnlimit: Number(this.f.Trdtxnlimit),
        Trdtotalshare: Number(this.f.Trdtotalshare),
        Trdagencyshare: Number(this.f.Trdagencyshare),
        Trdpayoutagentshare: Number(this.f.Trdpayoutagentshare),
        TrdcurrencyId: this.f.TrdcurrencyId,
      };
    }

    const serviceMethod = this.type === 'add' ? 'addAgent' : 'updateAgent';

    this.agentService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editAgent(agent: any, type: any) {
    if (agent?.Id) {
      this.type = type;
      this.editAgentById(agent?.Id);
    }
  }

  viewAgent(agent: any, type: any) {
    if (agent?.Id) {
      this.type = type;
      this.getAgentById(agent?.Id);
    }
  }

  getAgentById(agentId = 0) {
    this.agentService
      .getAgentById(
        getCacheData('admin_token'),
        getCacheData('username'),
        agentId
      )
      .subscribe({
        next: (res: any) => {
          const agentTypeId = this.agentTypeList.find(
            (p) => p.Group_Name == res?.[0]?.AgentType
          )?.Group_Id;

          this.getCountryInsert(agentTypeId, res);
        },
        error: (err) => {
          alert(err?.error?.message || 'Something went wrong');
        },
        complete: () => {},
      });
  }

  editAgentById(agentId = 0) {
    this.agentService.editAgentById(agentId).subscribe({
      next: (res: any) => {
        this.getCountryInsert(res?.[0]?.User_Group, res);
      },
      error: (err) => {
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }

  setFormValues(res) {
    let countryId,
      cityId,
      locationId,
      settlementCurrencyId,
      agentCurrencyId,
      domesticTransactionTypes,
      internationalTransactionTypes;
    if (this.type == 'view') {
      countryId = this.agentCountry.find(
        (p) => p.CountryName == res?.[0]?.CountryName
      )?.CountryId;

      cityId = this.cities.find(
        (p) => p.CityName == res?.[0]?.CityName
      )?.CityID;

      locationId = this.locations.find(
        (p) => p.LocationName == res?.[0]?.LocationName
      )?.LocationID;

      settlementCurrencyId = this.settlementCurrencies.find(
        (p) => p.CurrencyShortName == res?.[0]?.SettlementCurrency
      )?.Currency_Id;

      agentCurrencyId = this.agentCurrencies.find(
        (p) => p.CurrencyShortName == res?.[0]?.AgentCurrency
      )?.Currency_Id;

      domesticTransactionTypes = this.domesticTransactionTypes.find(
        (p) => p.CountryType == res?.[0]?.AllowedDomesticTransactionType
      )?.TypeList;
      internationalTransactionTypes = this.internationalTransactionTypes.find(
        (p) => p.CountryType == res?.[0]?.AllowedInternationalTransactionType
      )?.TypeList;
    } else {
      this.listofCurrencies = res?.[0]?.Currencies;
      this.trAmountLimits = res?.[0]?.CurrenciesTrAmtLimit;
      this.trAmountLimitBank = res?.[0]?.CurrenciesTrAmtLimit_Bank;
      this.trAmountLimitDraft = res?.[0]?.CurrenciesTrAmtLimit_DD;
      this.trAmountLimitWalt = res?.[0]?.CurrenciesTrAmtLimit_WALT;
      this.listofModeofPayments = res?.[0]?.ModeOfPayment;
    }
    countryId = this.agentCountry.find(
      (p) => p.CountryId == res?.[0]?.CountryID
    )?.CountryId;

    this.manageAgentForm.patchValue({
      AgentId: res?.[0]?.AgentID,
      TrdagentId: res?.[0]?.AgentType
        ? res?.[0]?.AgentType
        : res?.[0]?.User_Group,
      CountryID: countryId,
      CityID: cityId ? cityId : res?.[0]?.CityID,
      LocationID: locationId ? locationId : res?.[0]?.LocationID,
      Agent_Name: res?.[0]?.AgentName
        ? res?.[0]?.AgentName
        : res?.[0]?.Agent_Name,
      Agent_Add: res?.[0]?.Address ? res?.[0]?.Address : res?.[0]?.Agent_Add,
      ContactPerson: res?.[0]?.ContactPerson,
      Ph: res?.[0]?.Phone ? res?.[0]?.Phone : res?.[0]?.Ph,
      Fax: res?.[0]?.Fax,
      Email: res?.[0]?.Email,
      SettlementCurrency: settlementCurrencyId
        ? settlementCurrencyId
        : res?.[0]?.SettlementCurrency,
      Base_Currency_Id: agentCurrencyId
        ? agentCurrencyId
        : res?.[0]?.Currency_Id,
      CreditLimitInUSD: res?.[0]?.CrditLimitInSettlementCurrency
        ? res?.[0]?.CrditLimitInSettlementCurrency
        : res?.[0]?.CreditLimit,
      TELLimit: res?.[0]?.TrxnEnabledLimitTEL
        ? res?.[0]?.TrxnEnabledLimitTEL
        : res?.[0]?.TELLimit,
      AllowedDomesticTransactionType: domesticTransactionTypes
        ? domesticTransactionTypes
        : res?.[0]?.AllowedDomesticTransactionType,
      AllowedInternationalTransactionType: internationalTransactionTypes
        ? internationalTransactionTypes
        : res?.[0]?.AllowedInternationalTransactionType,
      NostroAcc: res?.[0]?.NostroAccount,
      NostroCode: res?.[0]?.NostroCode,
    });
    this.openAgentModal(this.type);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  addIndividualTransactionLimit() {
    if (this.f.Base_Currency_Id == 'None') {
      alert('Please Select Currency');
      return;
    }
    if (!this.f.CurrenciesTrAmtLimit) {
      alert('Please Fill Cash');
      return;
    }
    if (!this.f.CurrenciesTrAmtLimit_Bank) {
      alert('Please Fill Bank');
      return;
    }

    if (!this.f.CurrenciesTrAmtLimit_DD) {
      alert('Please Fill Draft');
      return;
    }
    if (!this.f.CurrenciesTrAmtLimit_WALT) {
      alert('Please Fill Walt');
      return;
    }

    const checkExistence = (array, value) =>
      array.some((item) => item.Value === value);

    const isCurrencyDifferentInList = !checkExistence(
      this.listofCurrencies,
      this.f.Base_Currency_Id
    );

    if (
      isCurrencyDifferentInList ||
      !checkExistence(this.trAmountLimits, this.f.CurrenciesTrAmtLimit) ||
      !checkExistence(
        this.trAmountLimitBank,
        this.f.CurrenciesTrAmtLimit_Bank
      ) ||
      !checkExistence(
        this.trAmountLimitDraft,
        this.f.CurrenciesTrAmtLimit_DD
      ) ||
      !checkExistence(this.trAmountLimitWalt, this.f.CurrenciesTrAmtLimit_WALT)
    ) {
      if (this.f.Base_Currency_Id != 'None') {
        const currencyName =
          this.agentCurrencies.find(
            (a) => a.Currency_Id == this.f.Base_Currency_Id
          )?.CurrencyShortName ?? '';
        this.addNewItem(this.listofCurrencies, 'Value', currencyName, 'Id');
      }

      if (this.f.CurrenciesTrAmtLimit?.length) {
        this.addNewItem(
          this.trAmountLimits,
          'Value',
          this.f.CurrenciesTrAmtLimit,
          'Id'
        );
      }
      if (this.f.CurrenciesTrAmtLimit_Bank?.length) {
        this.addNewItem(
          this.trAmountLimitBank,
          'Value',
          this.f.CurrenciesTrAmtLimit_Bank,
          'Id'
        );
      }
      if (this.f.CurrenciesTrAmtLimit_DD?.length) {
        this.addNewItem(
          this.trAmountLimitDraft,
          'Value',
          this.f.CurrenciesTrAmtLimit_DD,
          'Id'
        );
      }
      if (this.f.CurrenciesTrAmtLimit_WALT?.length) {
        this.addNewItem(
          this.trAmountLimitWalt,
          'Value',
          this.f.CurrenciesTrAmtLimit_WALT,
          'Id'
        );
      }
    } else {
      alert('This currency already exists in the list.');
      return;
    }
  }

  addNewItem(list, valueKey, newValue, idKey) {
    if (newValue != null && newValue !== '') {
      const numericValue = newValue;

      // Check if the value already exists in the list
      const isValueExists = list.some(
        (currency) => currency[valueKey] === numericValue
      );

      // If the value doesn't exist, add a new item
      if (!isValueExists) {
        let newCurrency = {
          [idKey]: list.length > 0 ? list[list.length - 1][idKey] + 1 : 1,
          [valueKey]: numericValue,
        };
        list.push(newCurrency);
      }
    }
  }

  removeIndividualTransactionLimit() {
    if (
      this.listofCurrencies?.length &&
      this.trAmountLimits?.length &&
      this.trAmountLimitBank?.length &&
      this.trAmountLimitDraft?.length &&
      this.trAmountLimitWalt?.length
    ) {
      this.listofCurrencies.pop();
      this.trAmountLimits.pop();
      this.trAmountLimitBank.pop();
      this.trAmountLimitDraft.pop();
      this.trAmountLimitWalt.pop();
      this.manageAgentForm.patchValue({
        CurrenciesTrAmtLimit: '',
        CurrenciesTrAmtLimit_Bank: '',
        CurrenciesTrAmtLimit_DD: '',
        CurrenciesTrAmtLimit_WALT: '',
      });
    }
  }

  addModeofPayment() {
    if (this.f.ModeOfPaymentID == 'All') {
      alert('Please Select Mode of Payment');
      return;
    }

    const checkExistence = (array, value) =>
      array.some((item) => item.Value === value);

    const isCurrencyDifferentInList = !checkExistence(
      this.listofModeofPayments,
      this.f.ModeOfPaymentID
    );

    if (isCurrencyDifferentInList) {
      this.addNewItem(
        this.listofModeofPayments,
        'Value',
        this.f.ModeOfPaymentID,
        'Id'
      );
    } else {
      alert('This mode of payment already exists in the list.');
      return;
    }
  }

  removeModeofPayment() {
    if (this.listofModeofPayments?.length) {
      this.listofModeofPayments.pop();
      this.manageAgentForm.patchValue({
        ModeOfPaymentID: 'All',
      });
    }
  }

  getRebateCurrencyType() {
    this.agentService
      .getRebateCurrencyType(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.rebateCurrencyType = res;
        },
        error: (err) => {
          this.rebateCurrencyType = [];
        },
        complete: () => {},
      });
  }
}
