import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { ViewComplianceListService } from '../services/view-compliance-list.service';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Component({
  selector: 'app-complaince-view-list-approve',
  templateUrl: './complaince-view-list-approve.component.html',
  styleUrls: ['./complaince-view-list-approve.component.css'],
})
export class ComplainceViewListApproveComponent implements OnInit {
  transaction: any;
  userId: any = 1;
  remark: string = '';
  Approveremark: string = '';
  transactionNumber: any;
  isMGTxn: any = 'eer';
  isMTOSuccess: any = 345;
  rejectReason: any = 'er';

  orgMsgID: any = 1;

  constructor(
    private route: ActivatedRoute,
    private viewComplianceListService: ViewComplianceListService,
    private http: HttpClient
  ) {}

  ngOnInit() {
    this.transactionNumber = this.route.snapshot.paramMap.get('id');

    this.viewComplianceListService
      .viewByTransactionId(this.transactionNumber, this.userId)
      .subscribe(
        (response) => {
          this.transaction = response;
        },
        (error) => {
          console.error(error);
        }
      );
  }

  // Method to update compliance on hold
  updateComplianceOnHold(
    transactionNumber: any = '',
    remark: any = '',
    userId: any = 1
  ) {
    const body = {};

    return this.http.patch(
      `${environment.apiUrl}/v1/api/compliance-management/onhold?TransactionNumber=${transactionNumber}&Remark=${remark}&UserID=${userId}`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  onHold() {
    this.updateComplianceOnHold(
      this.transactionNumber,
      this.remark,
      this.userId
    ).subscribe(
      (response) => {
        console.log('Compliance put on hold successfully:', response);

        this.remark = '';
      },
      (error) => {
        console.error('Error putting compliance on hold:', error);
      }
    );
  }

  onApprove() {
    this.viewComplianceListService
      .approveCompliance(
        this.transactionNumber,
        this.Approveremark,
        this.userId
      )
      .subscribe(
        (response) => {
          console.log('Compliance approved successfully:', response);
          // Handle success
          // Reset the remark after approval
          this.remark = '';
        },
        (error) => {
          console.error('Error approving compliance:', error);
          // Handle error
        }
      );
  }

  onReject() {
    this.viewComplianceListService
      .rejectCompliance(
        this.transactionNumber,
        this.isMGTxn,
        this.isMTOSuccess,
        this.rejectReason,
        this.userId,
        this.orgMsgID
        /* Pass appropriate values for isMGTxn, isMTOSuccess, rejectReason, and orgMsgID */
      )
      .subscribe(
        (response) => {
          console.log('Compliance rejected successfully:', response);
          // Handle success
          // Reset the remark after rejection
          this.remark = '';
        },
        (error) => {
          console.error('Error rejecting compliance:', error);
          // Handle error
        }
      );
  }
}
