import { Component, OnInit } from '@angular/core';
import { ComplianceReportService } from '../services/compliance-report.service';

@Component({
  selector: 'app-compliance-report',
  templateUrl: './compliance-report.component.html',
  styleUrls: ['./compliance-report.component.css'],
})
export class ComplianceReportComponent implements OnInit {
  filterData: any[] = [];
  complianceReportData: any[] = [];

  fromDate: string = '';
  toDate: string = '';
  constructor(private complianceService: ComplianceReportService) {}

  ngOnInit() {
    this.loadFilterData();
  }

  loadFilterData() {
    this.complianceService.getComplianceReportFilteredData().subscribe(
      (data: any[]) => {
        console.log(data);
        this.filterData = data;
      },
      (error) => {
        console.error(error);
      }
    );
  }

  selectedFilter: any;

  onFilterChange(event: any) {
    const selectedId = event.target.value;
    this.selectedFilter = selectedId;
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  loadComplianceReportData() {
    if (!this.fromDate || !this.toDate) {
      console.error('Please select both start date and end date.');
      return;
    }
    

    const fromDateObj = new Date(this.fromDate);
    const toDateObj = new Date(this.toDate);

    if (toDateObj < fromDateObj) {
      console.error('End date cannot be before start date.');
      return;
    }

    const formattedFromDate = fromDateObj.toISOString().split('T')[0];
    const formattedToDate = toDateObj.toISOString().split('T')[0];

    this.complianceService
      .getAllComplianceReportRecords(
        this.selectedFilter,
        formattedFromDate,
        formattedToDate
      )
      .subscribe(
        (data: any[]) => {
          this.complianceReportData = data;
        },
        (error) => {
          console.error(error);
        }
      );
  }
}
