import { Component, TemplateRef, ViewChild } from '@angular/core';
import { EditModeOfPaymentService } from '../services/edit-mode-of-payment.service';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';

@Component({
  selector: 'app-edit-mode-of-payment',
  templateUrl: './edit-mode-of-payment.component.html',
  styleUrls: ['./edit-mode-of-payment.component.css'],
})
export class EditModeOfPaymentComponent {
  @ViewChild('editModeofPayment') editModeofPaymentModal;
  bsModalRef: BsModalRef;

  countryList: any = [];
  selectedCountry: any = '-1';
  allAgentList: any = [];
  agentList: any = [];
  selectedAgent: any = '-1';
  modeofPaymentList: any = [];

  editModeofPayments: any = [];
  type: any = 'add';

  editModeofPaymentForm = new FormGroup({
    ID: new FormControl(),
    Agent_ID: new FormControl('None', Validators.required),
    ModeOfPayment_ID: new FormControl('None', Validators.required),
    Active: new FormControl(false),
  });

  constructor(
    private editModeofPaymentService: EditModeOfPaymentService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getModeofPaymentList();
  }

  getCountryList() {
    this.editModeofPaymentService.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0, agentName = '') {
    this.editModeofPaymentService.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.allAgentList = res;
        this.agentList = [...res];
        if (this.type == 'edit' && agentName) {
          const agentId = this.agentList.find(
            (c) => c.Agent == agentName
          )?.Agent_ID;
          this.editModeofPaymentForm.patchValue({
            Agent_ID: agentId,
          });
          this.openEditModeofPaymentModal('edit');
        }
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  getModeofPaymentList() {
    this.editModeofPaymentService.getModeofPaymentList().subscribe({
      next: (res: any) => {
        this.modeofPaymentList = res;
      },
      error: (err) => {
        this.modeofPaymentList = [];
      },
      complete: () => {},
    });
  }

  filter() {
    if (this.selectedCountry != '-1' && this.selectedAgent != '-1') {
      this.editModeofPaymentService
        .getFillGrid(this.selectedCountry, this.selectedAgent)
        .subscribe({
          next: (res: any) => {
            this.editModeofPayments = res;
          },
          error: (err) => {
            this.editModeofPayments = [];
          },
          complete: () => {},
        });
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openEditModeofPaymentModal(type: any) {
    if (type == 'add') {
      this.editModeofPaymentForm.reset();
    }
    this.type = type;
    this.openModal(this.editModeofPaymentModal);
  }

  get f() {
    return this.editModeofPaymentForm.value;
  }

  updateModeofPayment() {
    if (!this.editModeofPaymentForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    let payload;
    payload = {
      Agent_ID: this.f.Agent_ID,
      ModeOfPayment_ID: this.f.ModeOfPayment_ID,
      Active: this.f.Active,
      EnteredBy: 0,
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: Number(this.f.ID),
      };
    }

    this.editModeofPaymentService.updateEditModeofPayment(payload).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.editModeofPayments = [];
        this.selectedCountry = '-1';
        this.selectedAgent = '-1';
        alert('Agent wise Mode of Payment information updated successfully.');
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editMode(modeofPayment: any) {
    this.type = 'edit';

    const modeofPaymentId = this.modeofPaymentList.find(
      (p) => p.ModeOfPayment_Name == modeofPayment.ModeOfPayment_Name
    )?.ModeOfPayment_ID;

    const id = this.extractIdFromLink(modeofPayment.hLink);

    this.editModeofPaymentForm.patchValue({
      ID: id,
      ModeOfPayment_ID: modeofPaymentId,
      Active: modeofPayment.Active,
    });
    this.getAgentList(0, modeofPayment.Agent_Name);
  }

  private extractIdFromLink(link: string): string | null {
    const match = link.match(/[?&]ID=(\d+)/);
    return match ? match[1] : null;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
