import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { EditTransactionService } from '../services/edit-transaction.service';
import { getCacheData } from '../utils/helper';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'app-edit-transaction-detail',
  templateUrl: './edit-transaction-detail.component.html',
  styleUrls: ['./edit-transaction-detail.component.css'],
})
export class EditTransactionDetailComponent {
  transactionDetails: any = [];
  transactionStatus: any = [];
  destinationCity: any = [];
  recommendedAgent: any = [];
  idTypes: any = [];
  nationalities: any = [];
  branchId: any = '';
  appId: any = '';

  editTransactionForm = new FormGroup({
    TransactionNo: new FormControl('', Validators.required),
    txtRemark: new FormControl('', Validators.required),
    Beneficiary_Name_F: new FormControl('', Validators.required),
    Beneficiary_Name_L: new FormControl('', Validators.required),
    BAddress: new FormControl('', Validators.required),
    BMobile: new FormControl('', Validators.required),
    Customer_Name_F: new FormControl('', Validators.required),
    Customer_Name_L: new FormControl('', Validators.required),
    Address: new FormControl('', Validators.required),
    Mobile: new FormControl('', Validators.required),
    BankName: new FormControl('', Validators.required),
    BankBranch: new FormControl('', Validators.required),
    BankAccountNo: new FormControl('', Validators.required),
    txtBankCode: new FormControl('', Validators.required),
    ddlTransactionStatus: new FormControl('', Validators.required),
    ddlDestinationCity: new FormControl('', Validators.required),
    drpIDType: new FormControl('', Validators.required),
    hidLocId: new FormControl('', Validators.required),
    ddlRecommendedAgent: new FormControl('', Validators.required),
    IDNumber: new FormControl('', Validators.required),
    ExpDt: new FormControl('', Validators.required),
    DOB: new FormControl('', Validators.required),

    BPhone: new FormControl(''),
    HiddenMOPID: new FormControl(''),
    hidPayOutCountry: new FormControl(''),
    DestinationCountry: new FormControl(''),
    hidPayOutCCY: new FormControl(''),
    DestinationPayementMode: new FormControl(''),
    RecommendedAgent: new FormControl(''),
    AgentAddress: new FormControl(''),
    PurposeofTransfer: new FormControl(''),
    txtPurposeSpecify: new FormControl(''),
    HidTrType: new FormControl(''),
    hidTr_Display: new FormControl(''),
    AgentID: new FormControl(''),
    SubGroupID: new FormControl(''),
    ExchangeRate: new FormControl(''),
    PayInAmount: new FormControl(''),
    PayOutAmount: new FormControl(''),
    _PayOutAmount: new FormControl(''),
    _PayInAmount: new FormControl(''),
    lblIDRequired: new FormControl(''),
    CommissionCharge: new FormControl(''),
    TotalPayable: new FormControl(''),
    lblPayInCurrency: new FormControl(''),
    lblPayOutCurrency: new FormControl(''),
    lblCommissionCurrency: new FormControl(''),
    lblPaybleCurrency: new FormControl(''),
    TrDate: new FormControl(''),
    hidCust_Id: new FormControl(''),
    Phone: new FormControl(''),
    NationalityID: new FormControl('None'),
    HiddenCustomerDOB: new FormControl(''),
    Relation: new FormControl(''),
    txtPlaceOfBirth: new FormControl(''),
    BefZipCode: new FormControl(''),
    lblSuspected: new FormControl(true),
    hidBene_Id: new FormControl(''),
    BNationalityID: new FormControl(''),
    MessageToBen: new FormControl(''),
    MessageToAgent: new FormControl(''),
    LandMark: new FormControl(''),
    IsArebic: new FormControl(true),
    CourierAddress: new FormControl(''),
    CourierRemark: new FormControl(''),
    BankAddress: new FormControl(''),
    DDNumber: new FormControl(''),
    DDBank: new FormControl(''),
    DDBranch: new FormControl(''),
    DDBankAddress: new FormControl(''),
    UserID: new FormControl(''),
    TrTypeID: new FormControl(''),
    hidAppID: new FormControl(''),
    hidBranch: new FormControl(''),
    hidReceiveBranchID: new FormControl(''),
    HiddenExpDt: new FormControl(''),
  });

  constructor(
    private EditTrSvc: EditTransactionService,
    private route: ActivatedRoute
  ) {}

  ngOnInit(): void {
    this.branchId = this.route.snapshot.params['Branch'];
    this.appId = this.route.snapshot.params['AppId'];
    if (this.branchId && this.appId) {
      this.getTransactionDetails(this.branchId, this.appId);
    }
  }

  getTransactionDetails(branchId: any = 0, appId: any = 0) {
    this.EditTrSvc.getTransactionDetails(branchId, appId).subscribe({
      next: (res: any) => {
        this.transactionDetails = res;
        this.transactionStatus = res?.[0]?.status;
        this.destinationCity = res?.[0]?.city;
        this.recommendedAgent = res?.[0]?.getAgentLists;
        const transactionDate = this.formatDateForInput(
          new Date(res?.[0]?.TrDate)
        );
        this.idTypes = res?.[0]?.iDTypes;
        this.editTransactionForm.patchValue({
          TransactionNo: res?.[0]?.TansactionNo,
          DestinationCountry: res?.[0]?.DestinationCountry,
          ddlTransactionStatus: res?.[0]?.ddlTransactionStatusID,
          DestinationPayementMode: res?.[0]?.DestinationPayementMode,
          ddlDestinationCity: res?.[0]?.selectedCityID,
          ddlRecommendedAgent: res?.[0]?.ddlRecommendedAgent,
          AgentAddress: res?.[0]?.AgentAddress,
          PurposeofTransfer: res?.[0]?.PurposeofTransfer,
          txtPurposeSpecify: res?.[0]?.txtPurposeSpecify,
          TrDate: transactionDate,
          ExchangeRate: res?.[0]?.ExchangeRate,
          PayInAmount: res?.[0]?.PayInAmount,
          PayOutAmount: res?.[0]?.PayOutAmount,
          CommissionCharge: res?.[0]?.CommissionCharge,
          TotalPayable: res?.[0]?.TotalPayable,
          drpIDType: res?.[0]?.drpIDType,
          IDNumber: res?.[0]?.IDNumber,
          ExpDt: res?.[0]?.ExpDt,
          Customer_Name_F: res?.[0]?.Customer_Name_F,
          Customer_Name_L: res?.[0]?.Customer_Name_L,
          Address: res?.[0]?.Address,
          Mobile: res?.[0]?.Mobile,
          Phone: res?.[0]?.Phone,
          NationalityID: res?.[0]?.NationalityID,
          DOB: res?.[0]?.DOB,
          txtPlaceOfBirth: res?.[0]?.txtPlaceOfBirth,
          Relation: res?.[0]?.Relation,
          Beneficiary_Name_F: res?.[0]?.Beneficiary_Name_F,
          Beneficiary_Name_L: res?.[0]?.Beneficiary_Name_L,
          BAddress: res?.[0]?.BAddress,
          BMobile: res?.[0]?.BMobile,
          BPhone: res?.[0]?.BPhone,
          BNationalityID: res?.[0]?.BNationalityID,
          MessageToBen: res?.[0]?.MessageToBen,
          MessageToAgent: res?.[0]?.MessageToAgent,
          LandMark: res?.[0]?.LandMark,
          BefZipCode: res?.[0]?.BefZipCode,
          BankName: res?.[0]?.BankName,
          BankBranch: res?.[0]?.BankBranch,
          BankAddress: res?.[0]?.BankAddress,
          BankAccountNo: res?.[0]?.BankAccountNo,
          txtBankCode: res?.[0]?.txtBankCode,
          hidLocId: res?.[0]?.hidLocId,
          txtRemark: res?.[0]?.txtRemark,
        });
      },
      error: () => {
        this.transactionDetails = [];
      },
      complete: () => {},
    });
  }

  formatDateForInput(date: Date): string {
    return date.toISOString().split('T')[0];
  }

  formatDate(dateString: string): string {
    const date: Date = new Date(dateString);
    const parts: string[] = this.formatDateForInput(date).split('-');
    const formattedDay: string = parts[2];
    const formattedMonth: string = parts[1];
    const year: string = parts[0];

    return `${formattedDay}/${formattedMonth}/${year}`;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  get f() {
    return this.editTransactionForm.value;
  }

  updateTransfer() {
    if (!this.editTransactionForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.DOB == 'dd/MM/yyyy') {
      alert('Please Fill DOB!');
      return;
    }

    let payload;
    payload = {
      Address: this.f.Address,
      Mobile: this.f.Mobile,
      Phone: this.f.Phone ? this.f.Phone : '',
      Beneficiary_Name_F: this.f.Beneficiary_Name_F,
      Beneficiary_Name_L: this.f.Beneficiary_Name_L,
      BAddress: this.f.BAddress,
      BMobile: this.f.BMobile,
      BPhone: this.f.BMobile ? this.f.BMobile : '',
      BNationalityID: '0',
      Ex_Branch_ID: this.branchId,
      App_ID: this.appId,
      TrStatusID: this.f.ddlTransactionStatus,
      PayOutCityID: this.f.ddlDestinationCity,
      PayOutLocationID: this.f.hidLocId,
      PayOutAgent: this.f.ddlRecommendedAgent,
      BDO_Branch_ID: '5644',
      Remark: this.f.txtRemark,
      TrTypeID: '1',
      Type_ID: '6',
      IDNumber: this.f.IDNumber,
      IDExpDt: this.f.ExpDt,
      // IDExpDt:  '03/03/2024',
      EnteredBy: '1001000',
      DOB: this.formatDate(this.f.DOB),
      // DOB: '11/11/1980',
      BankAccountNo: this.f.BankAccountNo,
      BankName: this.f.BankName,
      BankCode: this.f.txtBankCode,
      BankBranch: this.f.BankBranch,
      BankAddress: this.f.BankAddress,
      TransactionNo: this.f.TransactionNo,
    };

    this.EditTrSvc.updateTransfer(payload).subscribe({
      next: (res: any) => {},
      error: (err) => {
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }
}
