import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { ManageCountryPermissionService } from '../services/manage-country-permission.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'app-manage-country-permission',
  templateUrl: './manage-country-permission.component.html',
  styleUrls: ['./manage-country-permission.component.css'],
})
export class ManageCountryPermissionComponent implements OnInit {
  @ViewChild('manageCountryPermission') manageCountryPermissionModal;
  bsModalRef: BsModalRef;

  baseCountries: any = [];
  selectedBaseCountry: any = 'All';

  targetCountries: any = [];
  selectedTargetCountry: any = 'All';

  allStatus: any = [
    { id: 1, name: 'Active' },
    { id: 0, name: 'Deactive' },
  ];
  selectedStatus: any = 1;

  type: any = 'add';

  countryPermissions: any = [];

  sendingPermissions: any = [];
  selectedSendingPermission: any = 'None';

  receivingPermissions: any = [];
  selectedReceivingPermission: any = 'None';

  permissions: any = [];
  selectedPermission: any = 2;

  countryPermissionEditData: any = [];

  manageCountryPermissionForm = new FormGroup({
    ID: new FormControl(),
    BaseCountryID: new FormControl('None', Validators.required),
    TargetCountryID: new FormControl('None', Validators.required),
    PermissionID: new FormControl('Select Permission', Validators.required),
    EnteredBy: new FormControl(''),
  });

  constructor(
    private countryPermission: ManageCountryPermissionService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getBaseCountries();
    this.getTargetCountries();
    this.getAllCountryPermission();
    this.getCountryPermission();
    this.getSendingCountryPermission();
    this.getReceivingCountryPermission();
  }

  getAllCountryPermission(baseCountryId = 0, targetCountryId = 0, active = 1) {
    this.countryPermission
      .getAllCountryPermission(
        getCacheData('admin_token'),
        getCacheData('username'),
        baseCountryId,
        targetCountryId,
        active
      )
      .subscribe({
        next: (res: any) => {
          this.countryPermissions = res;
        },
        error: (err) => {
          this.countryPermissions = [];
        },
        complete: () => {},
      });
  }

  getBaseCountries() {
    this.countryPermission
      .getBaseCountry(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.baseCountries = res;
        },
        error: (err) => {
          this.baseCountries = [];
        },
        complete: () => {},
      });
  }

  getTargetCountries(baseCountryId = 0) {
    this.countryPermission
      .getTargetCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        baseCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.targetCountries = res;
        },
        error: (err) => {
          this.targetCountries = [];
        },
        complete: () => {},
      });
  }

  onBaseCountryChange() {
    if (this.selectedBaseCountry != 'All') {
      this.getTargetCountries(this.selectedBaseCountry);
    } else {
      this.getTargetCountries();
    }
  }

  getSendingCountryPermission() {
    this.countryPermission
      .getSendingCountryPermission(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.sendingPermissions = res;
        },
        error: (err) => {
          this.sendingPermissions = [];
        },
        complete: () => {},
      });
  }

  getReceivingCountryPermission() {
    this.countryPermission.getReceivingCountryPermission().subscribe({
      next: (res: any) => {
        this.receivingPermissions = res;
      },
      error: (err) => {
        this.receivingPermissions = [];
      },
      complete: () => {},
    });
  }

  getCountryPermission() {
    this.countryPermission
      .getCountryPermission(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.permissions = res;
          this.manageCountryPermissionForm.patchValue({
            PermissionID: res?.[0]?.PermissionID,
          });
        },
        error: (err) => {
          this.permissions = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg', // 'modal-sm' for small, 'modal-lg' for large
    });
  }

  openCountryPermissionModal(type: any) {
    if (type == 'add') {
      this.manageCountryPermissionForm.reset({
        PermissionID: this.selectedPermission,
        BaseCountryID: this.selectedSendingPermission,
        TargetCountryID: this.selectedReceivingPermission,
      });
    }
    this.type = type;
    this.openModal(this.manageCountryPermissionModal);
  }

  searchCountryPermission() {
    let baseCountryId: number | null = 0;
    let targetCountryId: number | null = 0;

    if (this.selectedBaseCountry !== 'All') {
      baseCountryId = this.selectedBaseCountry;
    }

    if (this.selectedTargetCountry !== 'All') {
      targetCountryId = this.selectedTargetCountry;
    }

    this.getAllCountryPermission(
      baseCountryId,
      targetCountryId,
      this.selectedStatus
    );
  }

  get f() {
    return this.manageCountryPermissionForm.value;
  }

  addCountryPermission() {
    if (!this.manageCountryPermissionForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.BaseCountryID == 'None') {
      alert('Please Select Send Country Name!');
      return;
    }

    if (this.f.TargetCountryID == 'None') {
      alert('Please Select Receive Country Name!');
      return;
    }

    let payload;
    payload = {
      BaseCountryID: this.f.BaseCountryID,
      TargetCountryID: this.f.TargetCountryID,
      PermissionID: this.f.PermissionID,
      EnteredBy: this.f.EnteredBy || 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: this.f.ID,
        Active: true,
      };
    }

    const serviceMethod =
      this.type === 'add' ? 'addCountryPermission' : 'updateCountryPermission';

    this.countryPermission[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllCountryPermission();
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  editCountryPermission(countryPermission: any) {
    const permissionId = this.permissions.find(
      (p) => p.PermissionType == countryPermission.Permission
    )?.PermissionID;
    const baseCountryId =
      this.sendingPermissions.find(
        (p) => p.CountryName == countryPermission.BaseCountryName
      )?.CountryID || this.selectedSendingPermission;

    const targetCountryId = this.receivingPermissions.find(
      (p) => p.CountryName == countryPermission.TargetCountryName
    )?.CountryID;
    this.manageCountryPermissionForm.patchValue({
      ID: countryPermission?.ID,
      PermissionID: permissionId,
      BaseCountryID: baseCountryId,
      TargetCountryID: targetCountryId,
    });
    if (permissionId && baseCountryId) {
      this.getCountryPermissionList(permissionId, baseCountryId);
    }
    this.openCountryPermissionModal('edit');
  }

  getCountryPermissionList(permissionId = 0, baseCountryId = 0) {
    this.countryPermission
      .getCountryPermissionList(permissionId, baseCountryId)
      .subscribe({
        next: (res: any) => {
          this.countryPermissionEditData = res;
        },
        error: (err) => {
          this.countryPermissionEditData = [];
        },
        complete: () => {},
      });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
