import { Component } from '@angular/core';
import { ReceiveTransactionStatusSummaryService } from '../services/receive-transaction-status-summary.service';

@Component({
  selector: 'app-receive-transaction-status-summary',
  templateUrl: './receive-transaction-status-summary.component.html',
  styleUrls: ['./receive-transaction-status-summary.component.css'],
})
export class ReceiveTransactionStatusSummaryComponent {
  countryList: any = [];
  selectedCountry: any;
  agentList: any = [];
  selectedAgent: any;
  fromDate: any = '';
  toDate: any = '';
  statusList: any = [
    {
      id: 0,
      type: 'All',
    },
    {
      id: 2,
      type: 'Unpaid',
    },
    {
      id: 29,
      type: 'InProcess',
    },
    {
      id: 3,
      type: 'Paid',
    },
  ];
  selectedStatus: any = 0;
  allReceiveTransactions: any = [];
  today: string;
  minFromDate: string;
  isInvalidRange: boolean = false;

  constructor(
    private receiveTransactionSvc: ReceiveTransactionStatusSummaryService
  ) {
    this.today = new Date().toISOString().split('T')[0];
  }

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentList(0);
  }

  onSelectDates() {
    try {
      const startDateObj = new Date(this.fromDate);
      const endDateObj = new Date(this.toDate);

      const sixMonthsBeforeStartDate = new Date(startDateObj);
      sixMonthsBeforeStartDate.setMonth(
        sixMonthsBeforeStartDate.getMonth() - 6
      );

      if (endDateObj.getTime() !== sixMonthsBeforeStartDate.getTime()) {
        this.isInvalidRange = true;
      }
    } catch (error) {}
  }

  getCountryList() {
    this.receiveTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountry = res?.[0]?.CountryId;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.receiveTransactionSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.selectedAgent = res?.[0]?.Agent_ID;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.fromDate && !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }

    this.receiveTransactionSvc
      .getAllRecord(
        this.fromDate,
        this.toDate,
        this.selectedCountry,
        this.selectedAgent,
        this.selectedStatus
      )
      .subscribe({
        next: (res: any) => {
          this.allReceiveTransactions = res;
          if (this.allReceiveTransactions?.length) {
            this.allReceiveTransactions.forEach((item) => {
              if (typeof item === 'object' && item !== null) {
                item.agentname = this.sanitizeHTML(item.agentname);
              }
            });
          }
        },
        error: () => {
          this.allReceiveTransactions = [];
        },
        complete: () => {},
      });
  }

  sanitizeHTML(htmlString: string): string {
    const doc = new DOMParser().parseFromString(htmlString, 'text/html');
    return doc.body.textContent || '';
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
