import { Component, ElementRef, ViewChild } from '@angular/core';
import * as XLSX from 'xlsx';
import { SentTransactionSummaryService } from '../services/sent-transaction-summary.service';
import { ViewReceiveTransactionService } from '../services/view-receive-transaction.service';

@Component({
  selector: 'app-sent-transaction-summary',
  templateUrl: './sent-transaction-summary.component.html',
  styleUrls: ['./sent-transaction-summary.component.css'],
})
export class SentTransactionSummaryComponent {
  @ViewChild('sentTransactionTable') sentTransactionTable: ElementRef;

  countryList: any = [];
  selectedCountry: any = 0;
  agentList: any = [];
  selectedAgent: any;
  receiveCountryList: any = [];
  selectedReceiveCountry: any = 0;
  fromDate: any = '';
  toDate: any = '';
  modeType: any = [
    {
      id: -1,
      name: 'All',
    },
    {
      id: 2,
      name: 'Cash',
    },
    {
      id: 1,
      name: 'Bank',
    },
  ];
  selectedModeType: any = -1;
  allSentTransactionSummary: any = [];
  isInvalidRange: boolean = false;

  constructor(
    private sentTransactionSvc: SentTransactionSummaryService,
    private viewReceiveTransactionSvc: ViewReceiveTransactionService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentList(0);
    this.getReceiveCountryList();
  }

  getCountryList() {
    this.sentTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sentTransactionSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.selectedAgent = res?.[0]?.Agent_ID;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  getReceiveCountryList() {
    this.viewReceiveTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.receiveCountryList = res;
      },
      error: (err) => {
        this.receiveCountryList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.toDate && !this.fromDate) {
      alert('Please Select From & To Date!');
      return;
    }
    this.sentTransactionSvc
      .getAllRecord(
        this.fromDate,
        this.toDate,
        this.selectedModeType,
        this.selectedCountry,
        this.selectedAgent,
        this.selectedReceiveCountry
      )
      .subscribe({
        next: (res: any) => {
          this.allSentTransactionSummary = res;
        },
        error: () => {
          this.allSentTransactionSummary = [];
        },
        complete: () => {},
      });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.sentTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Sent_Transaction_Summary${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  onSelectDates() {
    try {
      const startDateObj = new Date(this.fromDate);
      const endDateObj = new Date(this.toDate);

      const sixMonthsBeforeStartDate = new Date(startDateObj);
      sixMonthsBeforeStartDate.setMonth(
        sixMonthsBeforeStartDate.getMonth() - 6
      );

      if (endDateObj.getTime() !== sixMonthsBeforeStartDate.getTime()) {
        this.isInvalidRange = true;
      }
    } catch (error) {}
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
