import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class BeneficiaryComplianceHoldService {
  constructor(private http: HttpClient) {}

  getAllBeneficiaryComplicance(BeneficiaryNo: any = 0): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });

    return this.http.get(
      `${environment.apiUrl}/v1/api/complicance/beneficiary-complicance-release/all-record?BeneficiaryNo=${BeneficiaryNo}`,
      { headers: headers }
    );
  }
  complicanceReleasView(BeneficiaryNo: string) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/complicance/beneficiary-complicance-release/view?BeneficiaryNo=${BeneficiaryNo}&status=Release`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  BeneficiaryComplicanceRelease(
    BeneficiaryNo: string,
    Remark: string,
    GUserName: string
  ) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });

    return this.http.post(
      `${environment.apiUrl}/v1/api/complicance/beneficiary-complicance-release/approve?Remark=${Remark}&BeneficiaryNo=${BeneficiaryNo}&GUserName=${GUserName}`,
      {}, // Empty request body
      { headers: headers }
    );
  }

  BeneficiaryComplicanceReject(
    BeneficiaryNo: string,
    Remark: string,
    GUserName: string
  ) {
    const headers = new HttpHeaders({
      Authorization: getCacheData('admin_token'),
      Accept: 'application/json',
      username: getCacheData('username'),
    });

    return this.http.post(
      `${environment.apiUrl}/v1/api/complicance/beneficiary-complicance-release/reject?Remark=${Remark}&BeneficiaryNo=${BeneficiaryNo}&GUserName=${GUserName}`,
      {}, // Empty request body
      { headers: headers }
    );
  }

  complicanceOfacLogDetails(TransactionNumber: string) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/complicance/beneficiary-complicance-release/ofac-log-details?TransactionNumber=${TransactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
