import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class ManageAgentBranchesService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getCountryList() {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/country`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentList(countryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/agent?CountryID=${countryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentBranchList(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/branch?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllRecord(agentId: any = 0, branchId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/get-all-record?AgentID=${agentId}&BranchID=${branchId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllRecordByBranchCode(branchCode: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/get-all-record-branch-code?BranchCode=${branchCode}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getCountryListByAgent(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/agent-wise-country?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentListByCountry(agentId: any = 0, countryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/country-wise-agent?AgentID=${agentId}&CountryID=${countryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getModeofPaymentByAgent(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/mode-of-payment?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getBranchCurrencyByAgent(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/branch-currency?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  addAgentBranch(payload: any = {}) {
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/add`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  viewAgentBranch(id: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/view?Id=${id}&ModuleID=73`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  approveAgentBranch(id: any = 0) {
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/approve?ID=${id}&ModuleID=73`,
      '',
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  editAgentBranch(branchId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/edit?BranchID=${branchId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  updateAgentBranch(payload: any = {}) {
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/manage-agent-branch/update`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
