import { Component, ElementRef, ViewChild } from '@angular/core';
import { ViewTransactionStatusService } from '../services/view-transaction-status.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-view-transaction-status',
  templateUrl: './view-transaction-status.component.html',
  styleUrls: ['./view-transaction-status.component.css'],
})
export class ViewTransactionStatusComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;

  sendCountries: any = [];
  selectedSendCountry: any = '-1';
  AgentList: any;
  DestCountryList: any = [];
  selectedAgent: any = 'All';
  agentBranches: any = [];
  AgentUser: any;
  selectedAgentBranch: any = 'All';
  status: any = [];
  selectedStatus: any = 'All';
  searchByType: any = 1;
  fromDate: any = '';
  toDate: any = '';
  transactionNo: any = '';
  allTransactionStatus: any = [];
  selectedFileFormat: any = '1';
  TransactionStatus: any;
  viewLastItem: any = -1;
  agentTypeList: any = [];
  selectedAgentType: any;
  sendCurrencyList: any = [];
  selectedSendCurrency: any = '-1';
  startDate: any;
  endDate: any;
  TransferTypeList: any;
  customerNameFirst: any;
  customerNameLast: any;
  beneficiaryNameFirst: any;
  beneficiaryNameLast: any;
  destCurrency: any;
  constructor(private viewTransactionSvc: ViewTransactionStatusService) {}

  ngOnInit(): void {
    this.getSendCountryList();
    this.getAgentTypeList();
    this.getTransactionStatus();
    this.getDestCountry();
  }

  getSendCountryList() {
    this.viewTransactionSvc.getSendCountryList().subscribe({
      next: (res: any) => {
        this.sendCountries = res;
        this.selectedSendCountry = res?.[0]?.CountryID;
        this.getSendCurrencyList(this.selectedSendCountry);
      },
      error: (err) => {
        this.sendCountries = [];
      },
      complete: () => {},
    });
  }

  onSendCountryChange(countryId: any) {
    this.getAgent(countryId);
    if (countryId != '-1') {
      this.getSendCurrencyList(countryId);
    }
  }

  getSendCurrencyList(sendCountryId) {
    this.viewTransactionSvc.getSendCurrencyList(sendCountryId).subscribe({
      next: (res: any) => {
        this.sendCurrencyList = res;
      },
      error: (err) => {
        this.sendCurrencyList = [];
      },
      complete: () => {},
    });
  }

  getDestCountry() {
    this.viewTransactionSvc.getDestCountry().subscribe({
      next: (res: any) => {
        this.DestCountryList = res;
      },
      error: (err) => {
        this.DestCountryList = [];
      },
      complete: () => {},
    });
  }

  onSelectDestCountry(countryId) {
    this.getDestCurrency(countryId);
  }

  onAgentChange(agentId: any) {
    if (agentId) {
      this.selectedAgent = agentId;
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId) {
    this.viewTransactionSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranches = res;
      },
      error: (err) => {
        this.agentBranches = [];
      },
      complete: () => {},
    });
  }

  onSelectAgentBranch(AgentBranchID) {
    this.getAgentUser(AgentBranchID, this.selectedAgent);
  }

  getDestCurrency(countryID) {
    this.viewTransactionSvc.getDestCurrency(countryID).subscribe({
      next: (res: any) => {
        this.destCurrency = res;
      },
      error: (err) => {
        this.destCurrency = [];
      },
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_Transaction_Status_${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  searchType(event) {
    this.searchByType = Number(event.target.value);
  }

  viewLast($event) {
    this.viewLastItem = $event.target.value;
  }

  getAgentTypeList() {
    this.viewTransactionSvc.getAgentTypeList().subscribe({
      next: (res: any) => {
        this.agentTypeList = res;
        this.selectedAgentType = res?.[0]?.Group_Id;
      },
      error: (err) => {
        this.agentTypeList = [];
      },
      complete: () => {
        this.getAgent(this.selectedSendCountry);
      },
    });

    this.getTransferTypeList();
  }

  getTransferTypeList(group: any = 0) {
    this.viewTransactionSvc
      .getTransferTypeList(this.selectedAgentType, group)
      .subscribe({
        next: (res: any) => {
          this.TransferTypeList = res;
        },
      });
  }

  getTransactionStatus() {
    this.viewTransactionSvc.getTransactionStatus().subscribe({
      next: (res: any) => {
        this.TransactionStatus = res;
      },
    });
  }

  getAgent(_TargetCountryID) {
    this.viewTransactionSvc.getAgentList(_TargetCountryID).subscribe({
      next: (res: any) => {
        this.AgentList = res;
        console.log(' res: ', res);
      },
    });
  }

  getAgentUser(AgentBranchID: any, AgentID: any) {
    this.viewTransactionSvc.getAgentUser(AgentBranchID, AgentID).subscribe({
      next: (res: any) => {
        this.AgentUser = res;
      },
    });
  }
}
